/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* Metal Pegboard Anchor
* Benjamen Johnson <3dprints.electronsmith.com>
* 20210331
*
* openSCAD Version: 2019.05 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
version="V1.1";

SQUARE=0;ROUNDED=1;
// add a little bit to cuts to make them render properly
$dl = 0.01;

/*[Pegboard Properties]*/
/*******************************************************************************
*
*******************************************************************************/
// Diameter of the pegboard holes
hole_dia = 6.25;

// Spacing between the pegboard holes
hole_spacing = 25.4;

/*[Anchor Properties]*/
/*******************************************************************************
*
*******************************************************************************/
// Number of columns to span
columns = 2; //[1:5]

// Number of rows to span
rows = 1;//[1:5]

// How much material do you want around the border?
border = 7;//[3:0.5:25.4]

// Should the corners be square or rounded
corners = 1; //[0:Square, 1:Rounded]

// Thickness of the base
base_thickness = 0.7;//[0.4:0.1:2]

// define the size of the base
base_height = (rows-1)*hole_spacing+2*border;
base_width = (columns-1)*hole_spacing+2*border;
BASE_DEF = [base_width,base_height,base_thickness];

/*[Screw Options]*/
/*******************************************************************************
* 
*******************************************************************************/
// Diameter of the screw hole 
screw_dia = 3;

// How far should anchor sick through the pegboard?
anchor_len = 5;

// Space between the anchor prongs
gap=0.25;

/*[Stamp Options]*/
/*******************************************************************************
* Stamp Options
*******************************************************************************/
// Which font do you want to use
font = "REGISTRATION PLATE UK:style=Bold";

// How big you want the text
font_size = 6;//[0:0.5:10 ]

// How thick the font is (back to front)
font_thickness = 0.2;

/*******************************************************************************
* Make it
*******************************************************************************/
x_start = -(columns-1)*hole_spacing/2;
y_start = -(rows-1)*hole_spacing/2;
anchor_pos = anchor_len/2+base_thickness;
START_POS = [x_start,y_start,0];

difference(){
    union(){
        if(corners == SQUARE)
            translate([0,0,base_thickness/2])
            cube(BASE_DEF,center=true);
            
        else{ //(corners == ROUND)
            translate([0,0,base_thickness/2])
            hull(){
                translate([x_start,y_start,0])
                cylinder(r=border,h=base_thickness,$fn=50,center=true);
                
                translate([-x_start, -y_start,0])
                cylinder(r=border,h=base_thickness,$fn=50,center=true);
                
                translate([x_start,-y_start,0])
                cylinder(r=border,h=base_thickness,$fn=50,center=true);
                
                translate([-x_start,y_start,0])
                cylinder(r=border,h=base_thickness,$fn=50,center=true);
            }// end hull
        }// end else
        
        for(x=[0:1:(columns-1)])
            for(y=[0:1:(rows-1)])
                translate(START_POS+[x*hole_spacing,y*hole_spacing,anchor_pos])
                cylinder(d=hole_dia,h=anchor_len,$fn=50, center=true);

    } //end union

    for(x=[0:1:(columns-1)])
        for(y=[0:1:(rows-1)])
            translate(START_POS+[x*hole_spacing,y*hole_spacing,0])
                cutout();
             
    translate([0,0,font_thickness-$dl])
    rotate([180,0,0])
    stamp_version(version, font_size, font_thickness);

} //end difference

/*******************************************************************************
* Module to cutout the cross hair hole
*******************************************************************************/
module cutout(){
    render()
    union(){
        translate([0,0,anchor_len/2])
        cube([hole_dia+$dl,gap,anchor_len*2],center=true);
            
        translate([0,0,anchor_len/2])
        cube([gap,hole_dia+$dl,anchor_len*2],center=true);
        
        cylinder(d1=screw_dia, d2=gap,h=anchor_len*3,$fn=50,center=true);
    } //end union    
} //end module

/*******************************************************************************
* Module for putting the version number on the print
*******************************************************************************/
module stamp_version(version_text,size,thickness){
    color("white")
    linear_extrude(height = thickness)
        text(text = version_text, font = font, size = size, valign="center",halign = "center");
} // end module
